// Copyright 1997 Ubilab, Union Bank of Switzerland. All rights reserved.
// This is part of the Geo system developed by Ubilab's DistrOOSE group.
// Check out: http://www.ubs.com/ubilab/Projects/se/se.html. GLAB-REH-97.

package Geo.Framework.AnyImpl;

import java.io.*;
import java.util.Vector;
import Geo.Framework.Any.*;



public abstract class AnyWriterDefImpl implements AnyWriter {

	// instance variables
	
	protected Vector anyBuffer;
	protected StringBuffer buffer;


	// constructors
	
	protected AnyWriterDefImpl() {
		anyBuffer = new Vector();
		buffer = new StringBuffer();
	}
	
	
	// accessing
	
	public abstract String getType();
	
	
	// writing
	
	public String writeAny(Any anAny) {
		addToBuffer("//Any:" + getType()); 	
		write(anAny);
		anyBuffer.removeAllElements();	
		return buffer.toString();					
	}	
	
	
	// protected - buffering
	
	protected void addToBuffer(String aString) {
		buffer.append(aString);
	}
	
	protected int anyBufferIndexOf(Any anAny){
		for (int i=0, s=anyBuffer.size(); i<s; i++) {
			if ( anyBuffer.elementAt(i) == anAny )
				return i;
		}
		return -1;
	}
	
	protected int buffer(Any anAny) {
		anyBuffer.addElement(anAny);
		return anyBuffer.size()-1;
	}
	
	
	// protected - dispatching/writing

	protected void dispatchWrite(Any anAny) {
		if (anAny instanceof AnyString) {
			writeAnyString(anAny.asAnyString());
			return;
		}
		if (anAny instanceof AnyInteger) {
			writeAnyInteger(anAny.asAnyInteger());
			return;
		}
		if (anAny instanceof AnyBoolean) {
			writeAnyBoolean(anAny.asAnyBoolean());
			return;
		}
		if (anAny instanceof AnyDouble) {
			writeAnyDouble(anAny.asAnyDouble());
			return;
		}
		if (anAny instanceof AnyArray) {
			writeAnyArray(anAny.asAnyArray());
			return;
		}
		if (anAny instanceof AnyDictionary) {
			writeAnyDictionary(anAny.asAnyDictionary());
			return;
		}
		if (anAny instanceof AnyFrame) {
			writeAnyFrame(anAny.asAnyFrame());
			return;
		}
		if (anAny instanceof AnyFrameDescriptor) {
			writeAnyFrameDescriptor(anAny.asAnyFrameDescriptor());
			return;
		}
		if (anAny instanceof AnySlotDescriptor) {
			writeAnySlotDescriptor(anAny.asAnySlotDescriptor());
			return;
		}
		if (anAny instanceof AnyNull || anAny == null) {
			writeNull();
			return;
		}
		throw new AnyWriterException(anAny);
	}

	protected abstract void write(Any anAny);
	protected abstract void writeAnyArray(AnyArray anAnyArray);
	protected abstract void writeAnyBoolean(AnyBoolean anAnyBoolean);
	protected abstract void writeAnyDictionary(AnyDictionary anAnyDictionary);
	protected abstract void writeAnyDouble(AnyDouble anAnyDouble);
	protected abstract void writeAnyFrame(AnyFrame anAnyFrame);
	protected abstract void writeAnyFrameDescriptor(AnyFrameDescriptor anAnyFrameDescriptor);
	protected abstract void writeAnyInteger(AnyInteger anAnyInteger);
	protected abstract void writeAnySlotDescriptor(AnySlotDescriptor anAnySlotDescriptor);
	protected abstract void writeAnyString(AnyString anAnyString);
	protected abstract void writeNull();
	
}
