// Copyright 1997 Ubilab, Union Bank of Switzerland. All rights reserved.
// This is part of the Geo system developed by Ubilab's DistrOOSE group.
// Check out: http://www.ubs.com/ubilab/Projects/se/se.html. GLAB-REH-97.

package Geo.Framework.AnyImpl;

import java.io.*;
import Geo.Framework.Any.*;



public class PrettyAnyWriterImpl extends AnyWriterDefImpl {

	// instance variables
	
	protected AnyFormatter formatter;


	// constructor
	
	public PrettyAnyWriterImpl() {
		formatter = new AnyFormatterImpl();
	}
	
	
	// accessing
	
	public String getType() {
		return "PrettyAnyIO";
	}
	
	
	// protected - writing
	
	protected void write(Any anAny) {
		// only composed anys are buffered
		if (anAny != null && !anAny.isSimple()) {
			int referencedId = this.anyBufferIndexOf(anAny);
			if ( referencedId == -1 ) {
				int id = this.buffer(anAny); // it was not in buffer	
				dispatchWrite(anAny);		
				addToBuffer(" &" + id); }
			else {
				addToBuffer(formatter.getIndentation());
				addToBuffer(" *" + referencedId); } }// it was in buffer
		else  // no type to be buffered
			dispatchWrite(anAny);
	}
	
	protected void writeAnyArray(AnyArray anAnyArray) {
		addToBuffer(formatter.getIndentation());
		addToBuffer("[" + anAnyArray.size());
		formatter.increment();
		if (anAnyArray.size()==0 
			|| anAnyArray.size()==1 && (anAnyArray.at(0)==null || anAnyArray.at(0).isSimple()) )
				formatter.setNoIndent();
		for (int i=0, s=anAnyArray.size(); i<s; i++) 
			write(anAnyArray.at(i));
		formatter.decrement();
		addToBuffer(formatter.getIndentation());
		addToBuffer("]");
		formatter.setDoIndent();
	}
	
	protected void writeAnyBoolean(AnyBoolean anAnyBoolean) {
		addToBuffer(formatter.getIndentation());
		addToBuffer(" " + anAnyBoolean);
	}
	
	protected void writeAnyDictionary(AnyDictionary anAnyDictionary) { 
		addToBuffer(formatter.getIndentation());
		addToBuffer("{"); 
		formatter.increment();
		for (int i=0, s=anAnyDictionary.size(); i<s; i++) {
			AnyString slotName = anAnyDictionary.keys().at(i).asAnyString();
			write(slotName);
			formatter.increment();
			write(anAnyDictionary.slotAt(slotName));
			formatter.decrement();
		}
		formatter.decrement();
		addToBuffer(formatter.getIndentation());
		addToBuffer("}");
	}
	
	protected void writeAnyDouble(AnyDouble anAnyDouble) {
		addToBuffer(formatter.getIndentation());
		addToBuffer(" D" + anAnyDouble);
	}
	
	protected void writeAnyFrame(AnyFrame anAnyFrame) {
		addToBuffer(formatter.getIndentation());
		addToBuffer(" <");
		formatter.increment();
		write(anAnyFrame.getFrameDescriptor().getName());
		for (int i=0, s=anAnyFrame.size(); i<s; i++) {
			AnyString slotName = anAnyFrame.slotNames().at(i).asAnyString();
			write(slotName);
			formatter.increment();
			write(anAnyFrame.slotAt(slotName));
			formatter.decrement();
		}
		formatter.decrement();
		addToBuffer(formatter.getIndentation());
		addToBuffer(">");
	}
	
	protected void writeAnyFrameDescriptor(AnyFrameDescriptor anAnyFrameDescriptor) {
		addToBuffer(formatter.getIndentation());
		addToBuffer(" #");
		formatter.increment(); 
		write(anAnyFrameDescriptor.getName());
		write(anAnyFrameDescriptor.getSuperFrameDescriptor());
		write(anAnyFrameDescriptor.getContents());
		formatter.decrement();
		addToBuffer(formatter.getIndentation());
		addToBuffer("#");
	}
	
	protected void writeAnyInteger(AnyInteger anAnyInteger) { 
		addToBuffer(formatter.getIndentation());
		addToBuffer(" " + anAnyInteger);
	}
	
	protected void writeAnySlotDescriptor(AnySlotDescriptor anAnySlotDescriptor) { 
		addToBuffer(formatter.getIndentation());
		addToBuffer(" ! ");
		addToBuffer(anAnySlotDescriptor.flagsToString());
		write(anAnySlotDescriptor.getTypeName());
	}
	
	protected void writeAnyString(AnyString anAnyString) { 
		StringBuffer tmp = new StringBuffer();
		addToBuffer(formatter.getIndentation());
		addToBuffer(" \"");
		for (int i=0, s=anAnyString.asString().length(); i<s; i++) {
			char c = anAnyString.asString().charAt(i);
			if (c == '\"' || c == '\\')
				tmp.append('\\');
			tmp.append(c);
		}
		addToBuffer(tmp.toString());
		addToBuffer("\"");
	}
	
	protected void writeNull() {
		addToBuffer(formatter.getIndentation());
		addToBuffer(" NULL");
	}
	
}
