// Copyright 1997 Ubilab, Union Bank of Switzerland. All rights reserved.
// This is part of the Geo system developed by Ubilab's DistrOOSE group.
// Check out: http://www.ubs.com/ubilab/Projects/se/se.html. GLAB-REH-97.

package Geo.Framework.AnyImpl;

import Geo.Framework.Any.*;



public class SimpleIndexManagerImpl implements IndexManager {

	// instance variables
	
	protected AnyDictionary indices;
	protected AnyDictionary frameIndices;


	// constructor
	
	public SimpleIndexManagerImpl() {
		indices = new AnyDictionaryImpl();
		frameIndices = new AnyDictionaryImpl();
	}
	
	
	// managing frame indices 
	
	public AnyArray getFrameIndex(AnyString key) {
		if ( !key.toString().startsWith("ALL_") )
			return null;
		AnyString indexName = AnySap.stringValue(key.toString().substring(4));
		if ( isFrameIndexEmpty(indexName) )
			return null;
		else
			return frameIndices.arrayAt(indexName);
	}
	
	public AnyArray getFrameIndex(String key) {
		return getFrameIndex(AnySap.stringValue(key));
	}		
	
	public void addFrameIndex(AnyString indexName) {
		frameIndices.setSlot(indexName, AnySap.nullValue());
	}
	
	public void addFrameIndex(String indexName) {
		addFrameIndex(AnySap.stringValue(indexName));
	}

	public void removeFrameIndex(AnyString indexName) {
		frameIndices.removeSlot(indexName);
	}
	
	public void removeFrameIndex(String indexName) {
		removeFrameIndex(AnySap.stringValue(indexName));
	}	
	
	
	// managing other indices

	public AnyArray getIndex(AnyString indexName) {
		return indices.arrayAt(indexName);
	}
	
	public AnyArray getIndex(String indexName) {
		return getIndex(AnySap.stringValue(indexName));
	}	
	
	public void addIndex(AnyString indexName, AnyArray index) {
		indices.setSlot(indexName, index);
	}
	
	public void addIndex(String indexName, AnyArray index) {
		addIndex(AnySap.stringValue(indexName), index);
	}
	
	public void removeIndex(AnyString indexName) {
		indices.removeSlot(indexName);
	}
	
	public void removeIndex(String indexName) {
		removeIndex(AnySap.stringValue(indexName));
	}
	
	
	// enrolling / unrolling
	
	public void enroll(AnyFrame anAnyFrame) {
		if ( !frameIndices.hasSlot(anAnyFrame.getTypeName()) )
			return;
		if ( isFrameIndexEmpty( anAnyFrame.getTypeName()) )
			frameIndices.setSlot(anAnyFrame.getTypeName(), anAnyFrame);
		else
			frameIndices.append(anAnyFrame.getTypeName(), anAnyFrame);
	}

	public void unroll(AnyFrame anAnyFrame) {
		if ( frameIndices.sizeAt(anAnyFrame.getTypeName()) == 0 )
			return;
		AnyArray index = frameIndices.arrayAt(anAnyFrame.getTypeName());
		for (int i=0, s=index.size(); i<s; i++) {
			if ( index.at(i) == anAnyFrame ) {
				index.remove(i);
				return;
			}
		}
	}		
	
	
	// printing

	public void printStatistics() {
		AnyArray frameIndicesNames = frameIndices.keys();
		for (int i=0, s=frameIndicesNames.size(); i<s; i++) {
			AnyString name = (AnyString) frameIndicesNames.at(i);
			int size;
			if (frameIndices.at(name) instanceof AnyNull)
				size = 0;
			else 
				size = frameIndices.sizeAt(name);
			System.out.println(name + " " + size );
		}
	}
	
	
	// misc
			
	public IndexManager duplicateWithSettings() {
		IndexManager duplicate = new SimpleIndexManagerImpl();
		AnyArray frameIndicesNames = frameIndices.keys();
		for (int i=0, s=frameIndicesNames.size(); i<s; i++)
			duplicate.addFrameIndex((AnyString) frameIndicesNames.at(i));
		return duplicate;
	}


	// private
	
	private boolean isFrameIndexEmpty(AnyString indexName) {
		return frameIndices.slotAt(indexName) instanceof AnyNull;
	}




}
